package com.yizhi.application.mq;


import cn.hutool.core.date.DateUtil;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.vod.model.v20170321.ListLiveRecordVideoRequest;
import com.aliyuncs.vod.model.v20170321.ListLiveRecordVideoResponse;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.gson.Gson;
import com.yizhi.application.domain.BizMessage;
import com.yizhi.application.enums.AliyunVideoStatusEnum;
import com.yizhi.application.service.IAliyunBizMessageService;
import com.yizhi.application.service.VideoInfoService;
import com.yizhi.application.util.PropertiesUtil;
import com.yizhi.application.vo.PlayInfoVo;
import com.yizhi.application.vo.VideoInfoVo;
import com.yizhi.application.vo.VideoPlayInfoVo;
import com.yizhi.core.application.constant.BizType;
import com.yizhi.live.application.enums.LiveDomainEnum;
import com.yizhi.live.application.feign.ApiLiveActivityFeign;
import com.yizhi.live.application.vo.LiveActivityVo;
import com.yizhi.video.application.feign.VideoClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.Objects;


@Component
@Slf4j
public class BizMessageDealJob {
	@Autowired
    private  IAliyunBizMessageService bizMessageService;
	@Autowired
    private DefaultAcsClient acsClient;
	@Autowired
    private PropertiesUtil propertiesUtil;
	@Autowired
    private VideoInfoService videoInfoService;
	@Autowired
    private ApiLiveActivityFeign liveActivityFeign;
	@Autowired
    private VideoClient videoClient;
    
    public void processBizMessage(Integer batchSize) {
    	LambdaQueryWrapper<BizMessage> condition = new LambdaQueryWrapper<BizMessage>()
        		.eq(BizMessage::getIsDeal, false)
        		.gt(BizMessage::getCreateTime, DateUtil.beginOfDay(new Date()))
        		.orderByAsc(BizMessage::getCreateTime);
    	
    	if( null != batchSize && !batchSize.equals(0)) {
    		condition.last("limit " + batchSize);
    	}
    	
        List<BizMessage> bizMessages = bizMessageService.list(condition);
        
        if (!CollectionUtils.isEmpty(bizMessages)) {
            bizMessages.parallelStream().forEach(a -> {
                switch (a.getEventType()) {
                    //视频获取回调url
                    case 1:
                    	updateVideoPlayUrl(a);
                    	break;
                    	
                    //直播获取录制视频回调url
                    case 2:
                        getLiveEndVideoUrl(a);
                        break;
                }
            });
        }
    }
    
    /**
     * 修改视频播放地址
     * @param bizMessage
     * @throws ClientException 
     * @throws ServerException 
     */
    private void updateVideoPlayUrl(BizMessage bizMessage) {
    	if (BizType.TYPE_COMMUNITY_VIDEO != bizMessage.getBizType()) {
            return;
        }
    	VideoInfoVo videoInfo;
		try {
			videoInfo = videoInfoService.getVideoInfo(bizMessage.getAliyunId());
    	
			log.info("[Video] get info from Aliyun:{}", videoInfo.toString());
	    	// status == normal 才能成功获取播放地址
	    	if(AliyunVideoStatusEnum.NORMAL.getValue().equals(videoInfo.getStatus())) {
	    		VideoPlayInfoVo playInfo = videoInfoService.getPlayUrl(bizMessage.getAliyunId());
	    		List<PlayInfoVo> playList = playInfo.getPlayInfoList();
	    		
	    		if(!CollectionUtils.isEmpty(playList)) {
	    			videoClient.updateVideoPlayUrl(bizMessage.getBizId(), playList.get(0).getPlayURL());
	    			bizMessage.setIsDeal(true);
	    			bizMessageService.updateById(bizMessage);
	    			log.info("[Video]BizMessageDealJob handled biz id:{}, biz type:{}, aliyun biz id:{}", bizMessage.getBizId(), bizMessage.getBizType(), bizMessage.getAliyunId());
	    		}
	    	}
		} catch (Exception e) {
			log.error("[Video]Biz message id {} failed. BizMessageDealJob has error:", bizMessage.getId(), e);
		}
    }

    /**
     * 查询到直播业务需要处理
     *
     * @param bizMessage
     */
    private void getLiveEndVideoUrl(BizMessage bizMessage) {
        if (BizType.TYPE_COMMUNITY_LIVE != bizMessage.getBizType()) {
            log.error("类型不匹配，拒绝执行,业务类型：" + bizMessage.getBizType());
            return;
        }
        //查询该流下面的回放list
        ListLiveRecordVideoRequest request = new ListLiveRecordVideoRequest();
        request.setAppName(propertiesUtil.LIVE_APP_NAME_VALUE);
        request.setDomainName(propertiesUtil.LIVE_DOMAIN);
        request.setStreamName(bizMessage.getBizId().toString());
        request.setSortBy("CreationTime:Desc");
        try {
            ListLiveRecordVideoResponse response = acsClient.getAcsResponse(request);
            log.info(new Gson().toJson(response));
            List<ListLiveRecordVideoResponse.LiveRecordVideo> liveRecordVideoList = response.getLiveRecordVideoList();
            if (!CollectionUtils.isEmpty(liveRecordVideoList)) {
                ListLiveRecordVideoResponse.LiveRecordVideo liveRecordVideo = liveRecordVideoList.get(0);
                ListLiveRecordVideoResponse.LiveRecordVideo.Video video = liveRecordVideo.getVideo();
                //状态正常再请求
                if (AliyunVideoStatusEnum.NORMAL.getValue().equals(video.getStatus())) {
                    String videoId = video.getVideoId();
                    VideoPlayInfoVo videoPlayInfoVo = videoInfoService.getPlayUrl(videoId);
                    if (!Objects.isNull(videoPlayInfoVo)) {
                        String playURL = videoPlayInfoVo.getPlayInfoList().get(0).getPlayURL();
                        String duration = videoPlayInfoVo.getPlayInfoList().get(0).getDuration();
                        liveActivityFeign.updateLiveUrl(new LiveActivityVo(bizMessage.getBizId(), playURL, duration));
                        log.info("直播：" + bizMessage.getBizId() + "回放url已获取");
                        bizMessage.setIsDeal(true);
            			bizMessageService.updateById(bizMessage);
                    }

                }
            }
        } catch (Exception e) {
        	log.error("[Live]Biz message id {} failed. BizMessageDealJob has error:", bizMessage.getId(), e);
        }
    }

}
